#!/usr/bin/python
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from charms.reactive import RelationBase
from charms.reactive import hook
from charms.reactive import scopes


class KeystoneCredentialsRequires(RelationBase):
    scope = scopes.GLOBAL

    # These remote data fields will be automatically mapped to accessors
    # with a basic documentation string provided.

    auto_accessors = [
        'api_version', 'auth_host', 'auth_port', 'auth_protocol',
        'credentials_host', 'credentials_password', 'credentials_port',
        'credentials_project', 'credentials_project_id',
        'credentials_protocol', 'credentials_username', 'domain', 'region'
    ]

    def update_state(self):
        if self.creds_data_complete():
            self.set_state('{relation_name}.available')
        else:
            self.remove_state('{relation_name}.available')

    @hook('{requires:keystone-credentials}-relation-changed')
    def changed(self):
        self.update_state()

    @hook('{requires:keystone-credentials}-relation-joined')
    def joined(self):
        self.set_state('{relation_name}.connected')
        self.update_state()

    @hook('{requires:keystone-credentials}-relation-{broken,departed}')
    def departed(self):
        self.update_state()

    def creds_data_complete(self):
        data = {
            'credentials_username': self.credentials_username(),
            'credentials_password': self.credentials_password(),
        }
        if all(data.values()):
            return True
        return False

    def configure(self, username):
        relation_info = {
            'username': username,
        }
        self.set_remote(**relation_info)
        self.set_local(**relation_info)

    def auth_url(self):
        auth_url_format_api_v = {
            "2": "{}://{}:{}/v2.0",
            "3": "{}://{}:{}/v3"
        }
        version = self.api_version()
        if version:
            return auth_url_format_api_v[version].format(
                self.credentials_protocol(),
                self.credentials_host(),
                self.credentials_port())

    def get_creds(self):
        "return all auto_accessors and auth_url {key,val}"
        return {key: getattr(self, key)()
                for key in self.auto_accessors + ['auth_url']}
