from charms.reactive import hook
from charms.reactive import RelationBase
from charms.reactive import scopes


class SwiftRequires(RelationBase):
    scope = scopes.UNIT

    @hook('{requires:swift}-relation-{joined,changed}')
    def changed(self):
        conv = self.conversation()
        if conv.get_remote('object_port'):
            # this unit's conversation has a port, so
            # it is part of the set of available units
            conv.set_state('{relation_name}.available')

    @hook('{requires:swift}-relation-{departed}')
    def departed(self):
        conv = self.conversation()
        conv.remove_state('{relation_name}.available')

    def services(self):
        """
        Returns a list of available HTTP services and their associated hosts
        and ports.
        The return value is a list of dicts of the following form::
            [
                {
                    'service_name': name_of_service,
                    'hosts': [
                        {
                            'hostname': address_of_host,
                            'object_port': object_port,
                            ...
                        },
                        # ...
                    ],
                },
                # ...
            ]
        """
        services = {}
        for conv in self.conversations():
            service_name = conv.scope.split('/')[0]
            service = services.setdefault(service_name, {
                'service_name': service_name,
                'hosts': [],
            })
            host = conv.get_remote('hostname') or \
                conv.get_remote('private-address')
            account_port = conv.get_remote('account_port')
            object_port = conv.get_remote('object_port')
            container_port = conv.get_remote('container_port')
            if host and account_port and object_port and container_port:
                service['hosts'].append({
                    'hostname': host,
                    'account_port': account_port,
                    'object_port': object_port,
                    'container_port': container_port,
                })
        return [s for s in services.values() if s['hosts']]
